@@ -1,12 +1,12 @@
 """Abstractions for pulling out 'field' definitions, which specify inputs, types, and
 defaults, from general callables."""
-
 from __future__ import annotations
 
 import collections
 import collections.abc
 import dataclasses
 import enum
+import functools
 import inspect
 import itertools
 import typing
@@ -14,13 +14,14 @@ import warnings
 from typing import (
     Any,
     Callable,
+    Dict,
     FrozenSet,
     Hashable,
     Iterable,
     List,
     Optional,
     Tuple,
-    Type,
+    TypeVar,
     Union,
     cast,
 )
@@ -31,26 +32,17 @@ from typing_extensions import get_args, get_type_hints, is_typeddict
 
 from . import conf  # Avoid circular import.
 from . import _docstrings, _instantiators, _resolver, _singleton, _strings
+from ._typing import TypeForm
 from .conf import _confstruct, _markers
 
-# Support attrs and pydantic if they're installed.
-try:
-    import attr
-except ImportError:
-    attr = None  # type: ignore
-try:
-    import pydantic
-except ImportError:
-    pydantic = None  # type: ignore
-
 
 @dataclasses.dataclass(frozen=True)
 class FieldDefinition:
     name: str
-    typ: Type
+    typ: TypeForm[Any]
     default: Any
     helptext: Optional[str]
-    markers: FrozenSet[_markers.Marker]
+    markers: FrozenSet[_markers._Marker]
 
     argconf: _confstruct._ArgConfiguration
 
@@ -69,12 +61,12 @@ class FieldDefinition:
     @staticmethod
     def make(
         name: str,
-        typ: Type,
+        typ: TypeForm[Any],
         default: Any,
         helptext: Optional[str],
         call_argname_override: Optional[Any] = None,
         *,
-        markers: Tuple[_markers.Marker, ...] = (),
+        markers: Tuple[_markers._Marker, ...] = (),
     ):
         # Try to extract argconf overrides from type.
         _, argconfs = _resolver.unwrap_annotated(typ, _confstruct._ArgConfiguration)
@@ -85,7 +77,7 @@ class FieldDefinition:
             (argconf,) = argconfs
             helptext = argconf.help
 
-        typ, inferred_markers = _resolver.unwrap_annotated(typ, _markers.Marker)
+        typ, inferred_markers = _resolver.unwrap_annotated(typ, _markers._Marker)
         return FieldDefinition(
             name if argconf.name is None else argconf.name,
             typ,
@@ -96,7 +88,7 @@ class FieldDefinition:
             call_argname_override if call_argname_override is not None else name,
         )
 
-    def add_markers(self, markers: Tuple[_markers.Marker, ...]) -> FieldDefinition:
+    def add_markers(self, markers: Tuple[_markers._Marker, ...]) -> FieldDefinition:
         return dataclasses.replace(
             self,
             markers=self.markers.union(markers),
@@ -170,10 +162,13 @@ class UnsupportedNestedTypeMessage:
     message: str
 
 
-def is_nested_type(typ: Type, default_instance: _DefaultInstance) -> bool:
+def is_nested_type(typ: TypeForm[Any], default_instance: _DefaultInstance) -> bool:
     """Determine whether a type should be treated as a 'nested type', where a single
     type can be broken down into multiple fields (eg for nested dataclasses or
-    classes)."""
+    classes).
+
+    TODO: we should come up with a better name than 'nested type', which is a little bit
+    misleading."""
     return not isinstance(
         _try_field_list_from_callable(typ, default_instance),
         UnsupportedNestedTypeMessage,
@@ -181,20 +176,46 @@ def is_nested_type(typ: Type, default_instance: _DefaultInstance) -> bool:
 
 
 def field_list_from_callable(
-    f: Union[Callable, Type],
+    f: Union[Callable, TypeForm[Any]],
     default_instance: _DefaultInstance,
-) -> List[FieldDefinition]:
+) -> Tuple[
+    Union[Callable, TypeForm[Any]], Dict[TypeVar, TypeForm], List[FieldDefinition]
+]:
     """Generate a list of generic 'field' objects corresponding to the inputs of some
-    annotated callable."""
-    out = _try_field_list_from_callable(f, default_instance)
+    annotated callable.
 
-    if isinstance(out, UnsupportedNestedTypeMessage):
-        raise _instantiators.UnsupportedTypeAnnotationError(out.message)
+    Returns:
+        The type that `f` is resolved as.
+        A type_from_typevar dict.
+        A list of field definitions.
+    """
+    # Resolve generic types.
+    f, type_from_typevar = _resolver.resolve_generic_types(f)
+    f = _resolver.narrow_type(f, default_instance)
+
+    # Try to generate field list.
+    field_list = _try_field_list_from_callable(f, default_instance)
+
+    if isinstance(field_list, UnsupportedNestedTypeMessage):
+        raise _instantiators.UnsupportedTypeAnnotationError(field_list.message)
 
     # Recursively apply markers.
-    _, parent_markers = _resolver.unwrap_annotated(f, _markers.Marker)
-    out = list(map(lambda field: field.add_markers(parent_markers), out))
-    return out
+    _, parent_markers = _resolver.unwrap_annotated(f, _markers._Marker)
+    field_list = list(map(lambda field: field.add_markers(parent_markers), field_list))
+
+    # Try to resolve types in our list of fields.
+    def resolve(field: FieldDefinition) -> FieldDefinition:
+        typ = field.typ
+        typ = _resolver.apply_type_from_typevar(typ, type_from_typevar)
+        typ = _resolver.type_from_typevar_constraints(typ)
+        typ = _resolver.narrow_container_types(typ, field.default)
+        typ = _resolver.narrow_union_type(typ, field.default)
+        field = dataclasses.replace(field, typ=typ)
+        return field
+
+    field_list = list(map(resolve, field_list))
+
+    return f, type_from_typevar, field_list
 
 
 # Implementation details below.
@@ -218,7 +239,7 @@ _known_parsable_types = set(
 
 
 def _try_field_list_from_callable(
-    f: Union[Callable, Type],
+    f: Union[Callable, TypeForm[Any]],
     default_instance: _DefaultInstance,
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     f, found_subcommand_configs = _resolver.unwrap_annotated(
@@ -230,79 +251,48 @@ def _try_field_list_from_callable(
     # Unwrap generics.
     f, type_from_typevar = _resolver.resolve_generic_types(f)
     f = _resolver.narrow_type(f, default_instance)
+    f_origin = _resolver.unwrap_origin_strip_extras(cast(TypeForm, f))
 
     # If `f` is a type:
     #     1. Set cls to the type.
     #     2. Consider `f` to be `cls.__init__`.
-    cls: Optional[Type] = None
+    cls: Optional[TypeForm[Any]] = None
     if isinstance(f, type):
         cls = f
         f = cls.__init__  # type: ignore
-        f_origin: Callable = cls  # type: ignore
-    f_origin = _resolver.unwrap_origin_strip_extras(f)
+        f_origin = cls  # type: ignore
 
-    # Try special cases.
+    # Try field generation from class inputs.
     if cls is not None:
-        if is_typeddict(cls):
-            return _try_field_list_from_typeddict(cls, default_instance)
+        for match, field_list_from_class in (
+            (is_typeddict, _field_list_from_typeddict),
+            (_resolver.is_namedtuple, _field_list_from_namedtuple),
+            (_resolver.is_dataclass, _field_list_from_dataclass),
+            (_is_attrs, _field_list_from_attrs),
+            (_is_pydantic, _field_list_from_pydantic),
+        ):
+            if match(cls):
+                return field_list_from_class(cls, default_instance)
 
-        if _resolver.is_namedtuple(cls):
-            return _try_field_list_from_namedtuple(cls, default_instance)
-
-        if _resolver.is_dataclass(cls):
-            return _try_field_list_from_dataclass(cls, default_instance)
-
-        if pydantic is not None and issubclass(cls, pydantic.BaseModel):
-            return _try_field_list_from_pydantic(cls, default_instance)
-
-        if attr is not None and attr.has(cls):
-            return _try_field_list_from_attrs(cls, default_instance)
-
-    # Standard container types. These are special because they can be nested structures
+    # Standard container types. These are different because they can be nested structures
     # if they contain other nested types (eg Tuple[Struct, Struct]), or treated as
     # single arguments otherwise (eg Tuple[int, int]).
     #
     # Note that f_origin will be populated if we annotate as `Tuple[..]`, and cls will
-    # be populated if we annotated as just `tuple`.
-    container_fields = None
+    # be populated if we annotate as just `tuple`.
     if f_origin is tuple or cls is tuple:
-        container_fields = _field_list_from_tuple(f, default_instance)
+        return _field_list_from_tuple(f, default_instance)
+    elif f_origin in (collections.abc.Mapping, dict) or cls in (
+        collections.abc.Mapping,
+        dict,
+    ):
+        return _field_list_from_dict(f, default_instance)
     elif f_origin in (list, set, typing.Sequence) or cls in (
         list,
         set,
         typing.Sequence,
     ):
-        contained_type: Any
-        if len(get_args(f)) == 0:
-            if default_instance in MISSING_SINGLETONS:
-                raise _instantiators.UnsupportedTypeAnnotationError(
-                    f"Sequence type {cls} needs either an explicit type or a"
-                    " default to infer from."
-                )
-            assert isinstance(default_instance, Iterable)
-            contained_type = next(iter(default_instance))
-        else:
-            (contained_type,) = get_args(f)
-        f_origin = list if f_origin is typing.Sequence else f_origin  # type: ignore
-
-        container_fields = _try_field_list_from_sequence(
-            contained_type, default_instance
-        )
-    elif f_origin in (collections.abc.Mapping, dict) or cls in (
-        collections.abc.Mapping,
-        dict,
-    ):
-        container_fields = _try_field_list_from_dict(f, default_instance)
-
-    # Check if one of the container types matched.
-    if container_fields is not None:
-        # Found fields!
-        if not isinstance(container_fields, UnsupportedNestedTypeMessage):
-            return container_fields
-        # Got an error,
-        else:
-            assert isinstance(container_fields, UnsupportedNestedTypeMessage)
-            return container_fields
+        return _field_list_from_sequence_checked(f, default_instance)
 
     # General cases.
     if (
@@ -313,8 +303,8 @@ def _try_field_list_from_callable(
         return _try_field_list_from_general_callable(f, cls, default_instance)
 
 
-def _try_field_list_from_typeddict(
-    cls: Type, default_instance: _DefaultInstance
+def _field_list_from_typeddict(
+    cls: TypeForm[Any], default_instance: _DefaultInstance
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     field_list = []
     valid_default_instance = (
@@ -345,8 +335,8 @@ def _try_field_list_from_typeddict(
     return field_list
 
 
-def _try_field_list_from_namedtuple(
-    cls: Type, default_instance: _DefaultInstance
+def _field_list_from_namedtuple(
+    cls: TypeForm[Any], default_instance: _DefaultInstance
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     # Handle NamedTuples.
     #
@@ -376,8 +366,8 @@ def _try_field_list_from_namedtuple(
     return field_list
 
 
-def _try_field_list_from_dataclass(
-    cls: Type, default_instance: _DefaultInstance
+def _field_list_from_dataclass(
+    cls: TypeForm[Any], default_instance: _DefaultInstance
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     # Handle dataclasses.
     field_list = []
@@ -405,14 +395,26 @@ def _try_field_list_from_dataclass(
     return field_list
 
 
-def _try_field_list_from_pydantic(
-    cls: Type, default_instance: _DefaultInstance
+# Support attrs and pydantic if they're installed.
+
+try:
+    import pydantic
+except ImportError:
+    pydantic = None  # type: ignore
+
+
+def _is_pydantic(cls: TypeForm[Any]) -> bool:
+    return pydantic is not None and issubclass(cls, pydantic.BaseModel)
+
+
+def _field_list_from_pydantic(
+    cls: TypeForm[Any], default_instance: _DefaultInstance
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     assert pydantic is not None
 
     # Handle pydantic models.
     field_list = []
-    for pd_field in cls.__fields__.values():
+    for pd_field in cls.__fields__.values():  # type: ignore
         field_list.append(
             FieldDefinition.make(
                 name=pd_field.name,
@@ -426,8 +428,18 @@ def _try_field_list_from_pydantic(
     return field_list
 
 
-def _try_field_list_from_attrs(
-    cls: Type, default_instance: _DefaultInstance
+try:
+    import attr
+except ImportError:
+    attr = None  # type: ignore
+
+
+def _is_attrs(cls: TypeForm[Any]) -> bool:
+    return attr is not None and attr.has(cls)
+
+
+def _field_list_from_attrs(
+    cls: TypeForm[Any], default_instance: _DefaultInstance
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     assert attr is not None
 
@@ -454,13 +466,13 @@ def _try_field_list_from_attrs(
 
 
 def _field_list_from_tuple(
-    f: Union[Callable, Type], default_instance: _DefaultInstance
+    f: Union[Callable, TypeForm[Any]], default_instance: _DefaultInstance
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     # Fixed-length tuples.
     field_list = []
     children = get_args(f)
     if Ellipsis in children:
-        return _try_field_list_from_sequence(
+        return _try_field_list_from_sequence_inner(
             next(iter(set(children) - {Ellipsis})), default_instance
         )
 
@@ -513,8 +525,25 @@ def _field_list_from_tuple(
     return field_list
 
 
-def _try_field_list_from_sequence(
-    contained_type: Type,
+def _field_list_from_sequence_checked(
+    f: Union[Callable, TypeForm[Any]], default_instance: _DefaultInstance
+) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
+    contained_type: Any
+    if len(get_args(f)) == 0:
+        if default_instance in MISSING_SINGLETONS:
+            raise _instantiators.UnsupportedTypeAnnotationError(
+                f"Sequence type {f} needs either an explicit type or a"
+                " default to infer from."
+            )
+        assert isinstance(default_instance, Iterable)
+        contained_type = next(iter(default_instance))
+    else:
+        (contained_type,) = get_args(f)
+    return _try_field_list_from_sequence_inner(contained_type, default_instance)
+
+
+def _try_field_list_from_sequence_inner(
+    contained_type: TypeForm[Any],
     default_instance: _DefaultInstance,
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     # When no default instance is specified:
@@ -557,8 +586,8 @@ def _try_field_list_from_sequence(
     return field_list
 
 
-def _try_field_list_from_dict(
-    f: Union[Callable, Type],
+def _field_list_from_dict(
+    f: Union[Callable, TypeForm[Any]],
     default_instance: _DefaultInstance,
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     if default_instance in MISSING_SINGLETONS:
@@ -581,8 +610,8 @@ def _try_field_list_from_dict(
 
 
 def _try_field_list_from_general_callable(
-    f: Union[Callable, Type],
-    cls: Optional[Type],
+    f: Union[Callable, TypeForm[Any]],
+    cls: Optional[TypeForm[Any]],
     default_instance: _DefaultInstance,
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
     # Handle general callables.
@@ -612,8 +641,26 @@ def _try_field_list_from_general_callable(
 
 
 def _field_list_from_params(
-    f: Union[Callable, Type], cls: Optional[Type], params: List[inspect.Parameter]
+    f: Union[Callable, TypeForm[Any]],
+    cls: Optional[TypeForm[Any]],
+    params: List[inspect.Parameter],
 ) -> Union[List[FieldDefinition], UnsupportedNestedTypeMessage]:
+    # Unwrap functools.wraps and functools.partial.
+    done = False
+    while not done:
+        done = True
+        if hasattr(f, "__wrapped__"):
+            f = f.__wrapped__
+            done = False
+        if isinstance(f, functools.partial):
+            f = f.func
+            done = False
+
+    # Sometime functools.* is applied to a class.
+    if isinstance(f, type):
+        cls = f
+        f = f.__init__  # type: ignore
+
     # Get type annotations, docstrings.
     docstring = inspect.getdoc(f)
     docstring_from_arg_name = {}
@@ -653,7 +700,7 @@ def _field_list_from_params(
         field_list.append(
             FieldDefinition.make(
                 name=param.name,
-                # Note that param.annotation does not resolve forward references.
+                # Note that param.annotation doesn't resolve forward references.
                 typ=hints[param.name],
                 default=default,
                 helptext=helptext,
