@@ -44,7 +44,6 @@ from typing import (
     List,
     Optional,
     Tuple,
-    Type,
     TypeVar,
     Union,
     cast,
@@ -54,6 +53,7 @@ from typing import (
 from typing_extensions import Annotated, Final, Literal, get_args, get_origin
 
 from . import _strings
+from ._typing import TypeForm
 
 _StandardInstantiator = Callable[[List[str]], Any]
 # Special case: the only time that argparse doesn't give us a string is when the
@@ -94,7 +94,7 @@ _builtin_set = set(
 
 
 def instantiator_from_type(
-    typ: Type, type_from_typevar: Dict[TypeVar, Type]
+    typ: TypeForm, type_from_typevar: Dict[TypeVar, TypeForm[Any]]
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     """Recursive helper for parsing type annotations.
 
@@ -152,8 +152,7 @@ def instantiator_from_type(
                 if i == 0 and param.annotation not in (str, inspect.Parameter.empty):
                     raise UnsupportedTypeAnnotationError(
                         f"Expected {typ} to be an `(arg: str) -> T` type converter, but"
-                        f" got {signature}. You may have a nested type in a container,"
-                        " which is unsupported."
+                        f" got {signature}."
                     )
                 if param.kind is inspect.Parameter.VAR_POSITIONAL:
                     has_var_positional = True
@@ -193,7 +192,7 @@ def instantiator_from_type(
         print(parser.parse_args().flag)
         ```
         """
-        assert len(get_args(typ)) == 0, f"Type {typ} cannot be instantiated."
+        assert len(get_args(typ)) == 0, f"TypeForm {typ} cannot be instantiated."
         (string,) = strings
         if typ is bool:
             return {"True": True, "False": False}[string]  # type: ignore
@@ -215,8 +214,8 @@ def instantiator_from_type(
 
 @overload
 def _instantiator_from_type_inner(
-    typ: Type,
-    type_from_typevar: Dict[TypeVar, Type],
+    typ: TypeForm,
+    type_from_typevar: Dict[TypeVar, TypeForm[Any]],
     allow_sequences: Literal["fixed_length"],
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     ...
@@ -224,8 +223,8 @@ def _instantiator_from_type_inner(
 
 @overload
 def _instantiator_from_type_inner(
-    typ: Type,
-    type_from_typevar: Dict[TypeVar, Type],
+    typ: TypeForm,
+    type_from_typevar: Dict[TypeVar, TypeForm[Any]],
     allow_sequences: Literal[False],
 ) -> Tuple[_StandardInstantiator, InstantiatorMetadata]:
     ...
@@ -233,16 +232,16 @@ def _instantiator_from_type_inner(
 
 @overload
 def _instantiator_from_type_inner(
-    typ: Type,
-    type_from_typevar: Dict[TypeVar, Type],
+    typ: TypeForm,
+    type_from_typevar: Dict[TypeVar, TypeForm[Any]],
     allow_sequences: Literal[True],
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     ...
 
 
 def _instantiator_from_type_inner(
-    typ: Type,
-    type_from_typevar: Dict[TypeVar, Type],
+    typ: TypeForm,
+    type_from_typevar: Dict[TypeVar, TypeForm[Any]],
     allow_sequences: Literal["fixed_length", True, False],
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     """Thin wrapper over instantiator_from_type, with some extra asserts for catching
@@ -261,7 +260,7 @@ def _instantiator_from_type_inner(
 
 
 def _instantiator_from_container_type(
-    typ: Type, type_from_typevar: Dict[TypeVar, Type]
+    typ: TypeForm, type_from_typevar: Dict[TypeVar, TypeForm[Any]]
 ) -> Optional[Tuple[Instantiator, InstantiatorMetadata]]:
     """Attempt to create an instantiator from a container type. Returns `None` is no
     container type is found."""
@@ -297,7 +296,7 @@ def _instantiator_from_container_type(
 
 
 def _instantiator_from_tuple(
-    typ: Type, type_from_typevar: Dict[TypeVar, Type]
+    typ: TypeForm, type_from_typevar: Dict[TypeVar, TypeForm[Any]]
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     types = get_args(typ)
     typeset = set(types)  # Note that sets are unordered.
@@ -381,7 +380,7 @@ def _join_union_metavars(metavars: Iterable[str]) -> str:
 
 
 def _instantiator_from_union(
-    typ: Type, type_from_typevar: Dict[TypeVar, Type]
+    typ: TypeForm, type_from_typevar: Dict[TypeVar, TypeForm[Any]]
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     options = list(get_args(typ))
     if NoneType in options:
@@ -457,7 +456,7 @@ def _instantiator_from_union(
 
 
 def _instantiator_from_dict(
-    typ: Type, type_from_typevar: Dict[TypeVar, Type]
+    typ: TypeForm, type_from_typevar: Dict[TypeVar, TypeForm[Any]]
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     key_type, val_type = get_args(typ)
     key_instantiator, key_meta = _instantiator_from_type_inner(
@@ -513,7 +512,7 @@ def _instantiator_from_dict(
 
 
 def _instantiator_from_sequence(
-    typ: Type, type_from_typevar: Dict[TypeVar, Type]
+    typ: TypeForm, type_from_typevar: Dict[TypeVar, TypeForm[Any]]
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     """Instantiator for variable-length sequences: list, sets, Tuple[T, ...], etc."""
     container_type = get_origin(typ)
@@ -557,7 +556,7 @@ def _instantiator_from_sequence(
 
 
 def _instantiator_from_literal(
-    typ: Type, type_from_typevar: Dict[TypeVar, Type]
+    typ: TypeForm, type_from_typevar: Dict[TypeVar, TypeForm[Any]]
 ) -> Tuple[Instantiator, InstantiatorMetadata]:
     choices = get_args(typ)
     str_choices = tuple(x.name if isinstance(x, enum.Enum) else str(x) for x in choices)
