@@ -29,6 +29,7 @@ from . import (
     _resolver,
     _strings,
 )
+from ._typing import TypeForm
 from .conf import _confstruct, _markers
 
 T = TypeVar("T")
@@ -58,7 +59,7 @@ class ParserSpecification:
     def from_callable_or_type(
         f: Callable[..., T],
         description: Optional[str],
-        parent_classes: Set[Type],
+        parent_classes: Set[Type[Any]],
         default_instance: Union[
             T, _fields.PropagatingMissingType, _fields.NonpropagatingMissingType
         ],
@@ -70,12 +71,13 @@ class ParserSpecification:
         # Consolidate subcommand types.
         consolidate_subcommand_args = (
             _markers.ConsolidateSubcommandArgs
-            in _resolver.unwrap_annotated(f, _markers.Marker)[1]
+            in _resolver.unwrap_annotated(f, _markers._Marker)[1]
         )
 
-        # Resolve generic types.
-        f, type_from_typevar = _resolver.resolve_generic_types(f)
-        f = _resolver.narrow_type(f, default_instance)
+        # Resolve the type of `f`, generate a field list.
+        f, type_from_typevar, field_list = _fields.field_list_from_callable(
+            f=f, default_instance=default_instance
+        )
 
         # Cycle detection.
         #
@@ -98,24 +100,7 @@ class ParserSpecification:
         subparsers = None
         subparsers_from_prefix = {}
 
-        field_list = _fields.field_list_from_callable(
-            f=f, default_instance=default_instance
-        )
         for field in field_list:
-            field = dataclasses.replace(
-                field,
-                # Resolve generic types.
-                typ=_resolver.narrow_container_types(
-                    _resolver.type_from_typevar_constraints(  # type: ignore
-                        _resolver.apply_type_from_typevar(
-                            field.typ,
-                            type_from_typevar,
-                        )
-                    ),
-                    default_instance=field.default,
-                ),
-            )
-
             if isinstance(field.typ, TypeVar):
                 raise _instantiators.UnsupportedTypeAnnotationError(
                     f"Field {field.name} has an unbound TypeVar: {field.typ}."
@@ -175,9 +160,9 @@ class ParserSpecification:
 
                     # Include nested subparsers.
                     if nested_parser.subparsers is not None:
-                        subparsers_from_prefix[
-                            nested_parser.subparsers.prefix
-                        ] = nested_parser.subparsers
+                        subparsers_from_prefix.update(
+                            nested_parser.subparsers_from_prefix
+                        )
                         subparsers = add_subparsers_to_leaves(
                             subparsers, nested_parser.subparsers
                         )
@@ -203,7 +188,8 @@ class ParserSpecification:
 
             # (3) Handle primitive or fixed types. These produce a single argument!
             arg = _arguments.ArgumentDefinition(
-                prefix=prefix,
+                dest_prefix=prefix,
+                name_prefix=prefix,
                 subcommand_prefix=subcommand_prefix,
                 field=field,
                 type_from_typevar=type_from_typevar,
@@ -291,11 +277,13 @@ class ParserSpecification:
         for arg in self.args:
             if (
                 arg.lowered.help is not argparse.SUPPRESS
-                and arg.prefix not in group_from_prefix
+                and arg.dest_prefix not in group_from_prefix
             ):
-                description = self.helptext_from_nested_class_field_name.get(arg.prefix)
-                group_from_prefix[arg.prefix] = parser.add_argument_group(
-                    format_group_name(arg.prefix),
+                description = self.helptext_from_nested_class_field_name.get(
+                    arg.dest_prefix
+                )
+                group_from_prefix[arg.dest_prefix] = parser.add_argument_group(
+                    format_group_name(arg.dest_prefix),
                     description=description,
                 )
 
@@ -305,8 +293,8 @@ class ParserSpecification:
                 arg.add_argument(positional_group)
                 continue
 
-            if arg.prefix in group_from_prefix:
-                arg.add_argument(group_from_prefix[arg.prefix])
+            if arg.dest_prefix in group_from_prefix:
+                arg.add_argument(group_from_prefix[arg.dest_prefix])
             else:
                 # Suppressed argument: still need to add them, but they won't show up in
                 # the helptext so it doesn't matter which group.
@@ -329,8 +317,8 @@ class SubparsersSpecification:
     @staticmethod
     def from_field(
         field: _fields.FieldDefinition,
-        type_from_typevar: Dict[TypeVar, Type],
-        parent_classes: Set[Type],
+        type_from_typevar: Dict[TypeVar, TypeForm[Any]],
+        parent_classes: Set[Type[Any]],
         prefix: str,
     ) -> Optional[SubparsersSpecification]:
         # Union of classes should create subparsers.
@@ -487,6 +475,10 @@ class SubparsersSpecification:
             ):
                 required = True
 
+        # Required if all args are pushed to the final subcommand.
+        if _markers.ConsolidateSubcommandArgs in field.markers:
+            required = True
+
         # Make description.
         description_parts = []
         if field.helptext is not None:
